@@ -36,20 +36,21 @@
 
 #define DSSTORE_VERSION 1
 
-#define DSSTORE_FLAGS_ACCESS_MASK		0x0001
-#define DSSTORE_FLAGS_ACCESS_READONLY	0x0000
-#define DSSTORE_FLAGS_ACCESS_READWRITE	0x0001
+#define DSSTORE_FLAGS_ACCESS_MASK      0x0001
+#define DSSTORE_FLAGS_ACCESS_READONLY  0x0000
+#define DSSTORE_FLAGS_ACCESS_READWRITE 0x0001
 
-#define DSSTORE_FLAGS_SERVER_MASK		0x0002
-#define DSSTORE_FLAGS_SERVER_CLONE		0x0000
-#define DSSTORE_FLAGS_SERVER_MASTER		0x0002
+#define DSSTORE_FLAGS_SERVER_MASK      0x0002
+#define DSSTORE_FLAGS_SERVER_CLONE     0x0000
+#define DSSTORE_FLAGS_SERVER_MASTER    0x0002
 
-#define DSSTORE_FLAGS_CACHE_MASK		0x0004
-#define DSSTORE_FLAGS_CACHE_ENABLED		0x0000
-#define DSSTORE_FLAGS_CACHE_DISABLED	0x0004
+#define DSSTORE_FLAGS_CACHE_MASK       0x0004
+#define DSSTORE_FLAGS_CACHE_ENABLED    0x0000
+#define DSSTORE_FLAGS_CACHE_DISABLED   0x0004
 
-#define DSSTORE_FLAGS_REMOTE_NETINFO	0x8000
-#define DSSTORE_FLAGS_OPEN_BY_TAG		0x4000
+#define DSSTORE_FLAGS_REMOTE_NETINFO   0x8000
+#define DSSTORE_FLAGS_OPEN_BY_TAG      0x4000
+#define DSSTORE_FLAGS_NOTIFY_CHANGES   0x0010
 
 typedef struct
 {
@@ -66,8 +67,12 @@ typedef struct
 	u_int32_t save_count;
 	u_int32_t remove_count;
 	u_int32_t index_count;
-	u_int32_t dirty;
+	u_int32_t file_info_count;
+	u_int32_t unused; /* removed "dirty" flag for 3423618 */
 	void **index;
+	void **file_info;
+	int notify_token;
+	char *notification_name;
 	void (*sync_delegate)(void *);
 	void *sync_private;
 } dsstore;
@@ -113,5 +118,7 @@ void dsstore_flush_cache(dsstore *s);
 void dsstore_reset(dsstore *s);
 
 void dsstore_set_sync_delegate(dsstore *, void (*)(void *), void *);
+void dsstore_set_notification_name(dsstore *, const char *);
+void dsstore_notify(dsstore *);
 
 #endif __DSSTORE_H__
