@@ -39,7 +39,6 @@
 #import <string.h>
 #import <stdlib.h>
 #import <sys/types.h>
-#import <sys/time.h>
 
 typedef struct {
 	LUCategory cat;
@@ -121,11 +120,59 @@ dictToDSRecord(LUDictionary *dict)
 
 	negative = NO;
 
+	memset(&(((_private_data *)_data)->access), 0, sizeof(struct timeval));
+
+	return self;
+}
+
+- (LUDictionary *)initTimeStamped
+{
+	char str[64];
+
+	[super init];
+	count = 0;
+	prop = (lu_property *)malloc(sizeof(lu_property) * (count + 1));
+
+	_data = (void *)malloc(sizeof(_private_data));
+
+	((_private_data *)_data)->cat = LUCategoryNull;
+	((_private_data *)_data)->ttl = 0;
+	((_private_data *)_data)->hits = 0;
+
+	sprintf(str, "D-0x%08x", (int)self);
+	[self setBanner:str];
+
+	negative = NO;
+
 	gettimeofday(&(((_private_data *)_data)->access), (struct timezone *)NULL);
 
 	return self;
 }
 
+- (LUDictionary *)initWithTime:(struct timeval)t
+{
+	char str[64];
+
+	[super init];
+	count = 0;
+	prop = (lu_property *)malloc(sizeof(lu_property) * (count + 1));
+
+	_data = (void *)malloc(sizeof(_private_data));
+
+	((_private_data *)_data)->cat = LUCategoryNull;
+	((_private_data *)_data)->ttl = 0;
+	((_private_data *)_data)->hits = 0;
+
+	sprintf(str, "D-0x%08x", (int)self);
+	[self setBanner:str];
+
+	negative = NO;
+
+	((_private_data *)_data)->access = t;
+
+	return self;
+}
+
 - (void)setCategory:(LUCategory)category
 {
 	((_private_data *)_data)->cat = category;
@@ -770,7 +817,27 @@ dictToDSRecord(LUDictionary *dict)
 	if (prop[where].val[0] == NULL) return 0;
 	return atoi(prop[where].val[0]);
 }
-	
+
+- (void)setInt:(int)i forKey:(char *)key
+{
+	char s[64];
+
+	if (key == NULL) return;
+
+	sprintf(s, "%d", i);
+	[self setValue:s forKey:key];
+}
+
+- (void)setUnsignedLong:(unsigned long)i forKey:(char *)key
+{
+	char s[64];
+
+	if (key == NULL) return;
+
+	sprintf(s, "%lu", i);
+	[self setValue:s forKey:key];
+}
+
 - (unsigned long)unsignedLongForKey:(char *)key
 {
 	unsigned int where;
@@ -834,6 +901,14 @@ dictToDSRecord(LUDictionary *dict)
 	return age;
 }
 
+- (time_t)dob
+{
+	time_t dob;
+
+	dob = ((_private_data *)_data)->access.tv_sec;
+	return dob;
+}
+
 - (char *)description
 {
 	unsigned int i, j, len, slen;
