@@ -50,7 +50,6 @@
 #import "LUDictionary.h"
 #import "MemoryWatchdog.h"
 #import "sys.h"
-#import <NetInfo/dns.h>
 #import <NetInfo/dsutil.h>
 #import <sys/file.h>
 #import <sys/types.h>
@@ -64,6 +63,8 @@
 #import <sys/time.h>
 #import <sys/resource.h>
 #import <signal.h>
+#import <notify.h>
+#import <mach/mig_errors.h>
 #import "_lu_types.h"
 
 #define forever for (;;)
@@ -72,7 +73,6 @@ extern int getppid(void);
 extern void interactive(FILE *, FILE*);
 
 extern int _lookup_link();
-extern int _lookup_one();
 extern int _lookup_all();
 
 #ifdef _UNIX_BSD_43_
@@ -98,11 +98,9 @@ syslock *rpcLock = NULL;
 syslock *statsLock = NULL;
 char *portName = NULL;
 sys_port_type server_port = SYS_PORT_NULL;
-#ifdef _SHADOW_
-sys_port_type server_port_privileged = SYS_PORT_NULL;
-sys_port_type server_port_unprivileged = SYS_PORT_NULL;
-BOOL shadow_passwords = NO;
-#endif
+BOOL debug_enabled = NO;
+BOOL statistics_enabled = NO;
+BOOL coredump_enabled = NO;
 
 /* Controller.m uses this global */
 BOOL shutting_down = NO;
@@ -111,6 +109,8 @@ static int configSource = configSourceAutomatic;
 static char *configPath = NULL;
 static char *configDomain = NULL;
 
+static int max_priority = -1;
+
 #define LONG_STRING_LENGTH 8192
 
 static void
@@ -158,38 +158,17 @@ detach(void)
 
 	setpgrp(0, getpid());
 #else
-	if (setsid() < 0) system_log(LOG_ERR, "lookupd: setsid() failed: %m");
+	if (setsid() < 0)
+	{
+		system_log(LOG_ERR, "lookupd: setsid() failed: %m");
+	}
 #endif
 }
 
-void
-parentexit(int x)
-{
-	exit(0);
-}
-
 void
 goodbye(int x)
 {
-	exit(1);
-}
-
-void
-handleSIGHUP()
-{
-	Thread *t;
-
-	/* Ignore HUP if already restarting */
-	if (shutting_down) return;
-
-	system_log(LOG_ERR, "Caught SIGHUP - restarting");
-	shutting_down = YES;
-
-	t = [[Thread alloc] init];
-	[t setName:"Knock Knock"];
-	[t setState:ThreadStateActive];
-	[t shouldTerminate:YES];
-	[t run:@selector(lookupdMessage) context:controller];
+	_exit(0);
 }
 
 static void
@@ -197,11 +176,25 @@ lookupd_startup()
 {
 	Thread *t;
 	BOOL status;
-	struct timeval tv;
 	char *name;
+	LUArray *config;
+	LUDictionary *cglobal;
+	int rf, nctoken;
+	uint32_t x;
+
+	rf = open("/dev/random", O_RDONLY, 0);
+	if (rf >= 0)
+	{
+		read(rf, &x, sizeof(uint32_t));
+		close(rf);
+	}
+	else
+	{
+		x = (getpid() << 10) + time(NULL);
+	}
+
+	srandom(x);
 
-	gettimeofday(&tv, NULL);
-	srandom((getpid() << 10) + tv.tv_usec);
 
 	rover = [[MemoryWatchdog alloc] init];
 
@@ -209,20 +202,52 @@ lookupd_startup()
 	[t setState:ThreadStateActive];
 
 	rpcLock = syslock_new(0);
+	syslock_set_name(rpcLock, RPCLockName);
+
 	statsLock = syslock_new(0);
+	syslock_set_name(statsLock, StatsLockName);
 
 	configManager = [[Config alloc] init];
 	status = [configManager setConfigSource:configSource path:configPath domain:configDomain];
 
+	if (max_priority == -1)
+	{
+		config = [configManager config];
+		cglobal = [configManager configGlobal:config];
+		max_priority = [configManager intForKey:"LogMaxPriority" dict:cglobal default:-1];
+		[config release];
+	}
+
 	cacheAgent = [[CacheAgent alloc] init];
 
-	name = portName;
-	if (portName == NULL) name = DefaultName;
+	name = NULL;
+	if (portName == NULL)
+	{
+		if (debugMode) name = strdup("lookupd-debug");
+		else name = strdup("lookupd");
+	}
+	else
+	{
+		if (debugMode)
+		{
+			name = malloc(strlen(portName) + 16);
+			sprintf(name, "lookupd.%s", portName);
+		}
+		else name = strdup("lookupd");
+	}
 	
 	system_log_open(name, (LOG_NOWAIT | LOG_PID), LOG_NETINFO, NULL);
+	if (name != NULL) free(name);
+
+	if (max_priority != -1) system_log_set_max_priority(max_priority);
 
 	controller = [[Controller alloc] initWithName:portName];
 
+	signal(SIGHUP, SIG_DFL);
+
+	nctoken = -1;
+	notify_register_signal(NETWORK_CHANGE_NOTIFICATION, SIGHUP, &nctoken);
+
 	if (!status)
 	{
 		if (debugMode)
@@ -245,16 +270,16 @@ lookupd_shutdown(int status)
 	[cacheAgent release];
 	cacheAgent = nil;
 
-	dns_shutdown();
-
 	[configManager release];
 	configManager = nil;
 
 	system_log(LOG_NOTICE, "lookupd exiting");
-	
+
+	syslock_set_name(rpcLock, NULL);
 	syslock_free(rpcLock);
 	rpcLock = NULL;
 
+	syslock_set_name(statsLock, NULL);
 	syslock_free(statsLock);
 	statsLock = NULL;
 
@@ -262,58 +287,7 @@ lookupd_shutdown(int status)
 
 	[rover release];
 
-	exit(status);
-}
-
-/*
- * Restart everything.
- */
-void
-restart()
-{
-#ifdef _SHADOW_
-	char *Argv[7], portstr2[32];
-#else
-	char *Argv[5];
-#endif
-	char pidstr[32], portstr1[32];
-	int pid;
-
-	if (debugMode) lookupd_shutdown(0);
-
-	system_log(LOG_NOTICE, "Restarting lookupd");
-
-#ifdef _SHADOW_
-	sprintf(pidstr,  "%d", getpid());
-	sprintf(portstr1, "%d", server_port_unprivileged);
-	sprintf(portstr2, "%d", server_port_privileged);
-
-	Argv[0] = "lookupd";
-	Argv[1] = "-r";
-	Argv[2] = portstr1;
-	Argv[3] = portstr2;
-	Argv[4] = pidstr;
-	Argv[5] = shadow_passwords ? NULL : "-u";
-	Argv[6] = NULL;
-#else
-	sprintf(pidstr,  "%d", getpid());
-	sprintf(portstr1, "%d", server_port);
-
-	Argv[0] = "lookupd";
-	Argv[1] = "-r";
-	Argv[2] = portstr1;
-	Argv[3] = pidstr;
-	Argv[4] = NULL;
-#endif
-
-	pid = fork();
-	if (pid > 0)
-	{
-		signal(SIGTERM, parentexit);
-		forever [[Thread currentThread] sleep:1];
-	}
-
-	execv(EXE_FILE, Argv);
+	exit(0);
 }
 
 int
@@ -371,7 +345,7 @@ query_util(char *str)
 	XDR outxdr;
 	XDR inxdr;
 	int proc;
-	unit lookup_buf[MAX_INLINE_UNITS * BYTES_PER_XDR_UNIT];
+	char *lookup_buf;
 	char databuf[_LU_MAXLUSTRLEN * BYTES_PER_XDR_UNIT];
 	int n, i;
 	kern_return_t status;
@@ -407,11 +381,11 @@ query_util(char *str)
 		return;
 	}
 
-	datalen = MAX_INLINE_UNITS * BYTES_PER_XDR_UNIT;
+	datalen = 0;
+	lookup_buf = NULL;
 
-	if (_lookup_one(server_port, proc, (unit *)databuf,
-		xdr_getpos(&outxdr) / BYTES_PER_XDR_UNIT, lookup_buf, &datalen)
-		!= KERN_SUCCESS)
+	status = _lookup_all(server_port, proc, (unit *)databuf, xdr_getpos(&outxdr) / BYTES_PER_XDR_UNIT, &lookup_buf, &datalen);
+	if (status != KERN_SUCCESS)
 	{
 		xdr_destroy(&outxdr);
 		fprintf(stderr, "lookup failed!\n");
@@ -421,6 +395,11 @@ query_util(char *str)
 	xdr_destroy(&outxdr);
 
 	datalen *= BYTES_PER_XDR_UNIT;
+	if ((lookup_buf == NULL) || (datalen == 0))
+	{
+		fprintf(stderr, "lookup returned NULL!\n");
+		return;
+	}
 
 	xdrmem_create(&inxdr, lookup_buf, datalen, XDR_DECODE);
 
@@ -428,6 +407,7 @@ query_util(char *str)
 	if (!xdr_int(&inxdr, &n))
 	{
 		xdr_destroy(&inxdr);
+		vm_deallocate(mach_task_self(), (vm_address_t)lookup_buf, datalen);
 		fprintf(stderr, "xdr decoding error!\n");
 		return;
 	}
@@ -440,6 +420,7 @@ query_util(char *str)
 	if (n > 0) printf("\n");
 
 	xdr_destroy(&inxdr);
+	vm_deallocate(mach_task_self(), (vm_address_t)lookup_buf, datalen);
 }
 
 void
@@ -449,7 +430,7 @@ query_find(char *cat, char *key, char *val)
 	XDR outxdr;
 	XDR inxdr;
 	int proc;
-	unit lookup_buf[MAX_INLINE_UNITS * BYTES_PER_XDR_UNIT];
+	char *lookup_buf;
 	char databuf[_LU_MAXLUSTRLEN * BYTES_PER_XDR_UNIT];
 	int n, i;
 	kern_return_t status;
@@ -459,7 +440,6 @@ query_find(char *cat, char *key, char *val)
 	status = KERN_SUCCESS;
 
 	status = _lookup_link(server_port, "find", &proc);
-
 	if (status != KERN_SUCCESS)
 	{
 		fprintf(stderr, "can't find lookup procedure\n");
@@ -475,8 +455,6 @@ query_find(char *cat, char *key, char *val)
 		return;
 	}
 
-	datalen = MAX_INLINE_UNITS * BYTES_PER_XDR_UNIT;
-
 	if (!xdr__lu_string(&outxdr, &key))
 	{
 		xdr_destroy(&outxdr);
@@ -491,9 +469,11 @@ query_find(char *cat, char *key, char *val)
 		return;
 	}
 
-	if (_lookup_one(server_port, proc, (unit *)databuf,
-		xdr_getpos(&outxdr) / BYTES_PER_XDR_UNIT, lookup_buf, &datalen)
-		!= KERN_SUCCESS)
+	datalen = 0;
+	lookup_buf = NULL;
+
+	status = _lookup_all(server_port, proc, (unit *)databuf, xdr_getpos(&outxdr) / BYTES_PER_XDR_UNIT, &lookup_buf, &datalen);
+	if (status != KERN_SUCCESS)
 	{
 		xdr_destroy(&outxdr);
 		fprintf(stderr, "lookup failed!\n");
@@ -503,13 +483,18 @@ query_find(char *cat, char *key, char *val)
 	xdr_destroy(&outxdr);
 
 	datalen *= BYTES_PER_XDR_UNIT;
+	if ((lookup_buf == NULL) || (datalen == 0))
+	{
+		fprintf(stderr, "lookup returned NULL!\n");
+		return;
+	}
 
 	xdrmem_create(&inxdr, lookup_buf, datalen, XDR_DECODE);
 
-
 	if (!xdr_int(&inxdr, &n))
 	{
 		xdr_destroy(&inxdr);
+		vm_deallocate(mach_task_self(), (vm_address_t)lookup_buf, datalen);
 		fprintf(stderr, "xdr decoding error!\n");
 		return;
 	}
@@ -522,6 +507,7 @@ query_find(char *cat, char *key, char *val)
 	if (n > 0) printf("\n");
 
 	xdr_destroy(&inxdr);
+	vm_deallocate(mach_task_self(), (vm_address_t)lookup_buf, datalen);
 }
 
 void
@@ -533,9 +519,24 @@ query_query(int argc, char *argv[])
 	int proc;
 	char *listbuf;
 	char databuf[_LU_MAXLUSTRLEN * BYTES_PER_XDR_UNIT];
-	int n, i, j, na;
+	int n, i, j, na, cat;
 	kern_return_t status;
-	char *k;
+	char *k, str[16], *procname;
+
+	procname = "query";
+
+	/* check category */
+	cat = -1;
+	if (argv[1][0] != '-')
+	{
+		cat = [LUAgent categoryWithName:argv[1]];
+		if ((cat < 0) || (cat > NCATEGORIES))
+		{
+			fprintf(stderr, "invalid category %s\n", argv[1]);
+			return;
+		}
+	}
+	else procname = argv[1] + 1;
 
 	/* check the "-a" options */
 
@@ -572,15 +573,17 @@ query_query(int argc, char *argv[])
 	}
 
 	server_port = lookupd_port(portName);
-	status = _lookup_link(server_port, "query", &proc);
+	status = _lookup_link(server_port, procname, &proc);
 	if (status != KERN_SUCCESS)
 	{
-		fprintf(stderr, "can't find query procedure\n");
+		fprintf(stderr, "can't find %s procedure\n", procname);
 		return;
 	}
 
 	xdrmem_create(&outxdr, databuf, sizeof(databuf), XDR_ENCODE);
 
+	if (cat == -1) na--;
+
 	/* Encode attribute count */
 	if (!xdr_int(&outxdr, &na))
 	{
@@ -589,27 +592,36 @@ query_query(int argc, char *argv[])
 		return;
 	}
 
-	/* Encode "_lookup_category" attribute */
-	k = copyString("_lookup_category");
-	if (!xdr__lu_string(&outxdr, &k))
-	{
-		xdr_destroy(&outxdr);
-		fprintf(stderr, "xdr encoding error!\n");
-		return;
-	}
-	free(k);
-	n = 1;
-	if (!xdr_int(&outxdr, &n))
-	{
-		xdr_destroy(&outxdr);
-		fprintf(stderr, "xdr encoding error!\n");
-		return;
-	}
-	if (!xdr__lu_string(&outxdr, &argv[1]))
-	{
-		xdr_destroy(&outxdr);
-		fprintf(stderr, "xdr encoding error!\n");
-		return;
+	if (cat != -1)
+	{ 
+		/* Encode "_lookup_category" attribute */
+		k = copyString("_lookup_category");
+		if (!xdr__lu_string(&outxdr, &k))
+		{
+			xdr_destroy(&outxdr);
+			fprintf(stderr, "xdr encoding error!\n");
+			free(k);
+			return;
+		}
+
+		free(k);
+
+		n = 1;
+		if (!xdr_int(&outxdr, &n))
+		{
+			xdr_destroy(&outxdr);
+			fprintf(stderr, "xdr encoding error!\n");
+			return;
+		}
+
+		sprintf(str, "%d", cat);
+		k = str;
+		if (!xdr__lu_string(&outxdr, &k))
+		{
+			xdr_destroy(&outxdr);
+			fprintf(stderr, "xdr encoding error!\n");
+			return;
+		}
 	}
 
 	for (i = 2; i < argc; i++)
@@ -644,9 +656,8 @@ query_query(int argc, char *argv[])
 	listbuf = NULL;
 	datalen = 0;
 
-	if (_lookup_all(server_port, proc, (unit *)databuf,
-		xdr_getpos(&outxdr) / BYTES_PER_XDR_UNIT, &listbuf, &datalen)
-		!= KERN_SUCCESS)
+	status = _lookup_all(server_port, proc, (unit *)databuf, xdr_getpos(&outxdr) / BYTES_PER_XDR_UNIT, &listbuf, &datalen);
+	if (status != KERN_SUCCESS)
 	{
 		xdr_destroy(&outxdr);
 		fprintf(stderr, "query failed!\n");
@@ -655,9 +666,7 @@ query_query(int argc, char *argv[])
 
 	xdr_destroy(&outxdr);
 
-#ifdef _IPC_TYPED_
 	datalen *= BYTES_PER_XDR_UNIT;
-#endif
 	
 	xdrmem_create(&inxdr, listbuf, datalen, XDR_DECODE);
 
@@ -699,16 +708,16 @@ query_list(char *str)
 	encode_len = 0;
 
 	status = _lookup_link(server_port, "list", &proc);
-	listproc = 1;
-
-	if (streq(str, "-configuration")) str = "config";
-
 	if (status != KERN_SUCCESS)
 	{
 		fprintf(stderr, "can't find lookup procedure\n");
 		return;
 	}
-	
+
+	listproc = 1;
+
+	if (streq(str, "-configuration")) str = "config";
+
 	xdrmem_create(&outxdr, databuf, sizeof(databuf), XDR_ENCODE);
 	
 	if (!xdr__lu_string(&outxdr, &str))
@@ -723,9 +732,8 @@ query_list(char *str)
 	listbuf = NULL;
 	datalen = 0;
 
-	if (_lookup_all(server_port, proc, (unit *)databuf,
-		xdr_getpos(&outxdr) / BYTES_PER_XDR_UNIT, &listbuf, &datalen)
-		!= KERN_SUCCESS)
+	status = _lookup_all(server_port, proc, (unit *)databuf, xdr_getpos(&outxdr) / BYTES_PER_XDR_UNIT, &listbuf, &datalen);
+	if (status != KERN_SUCCESS)
 	{
 		xdr_destroy(&outxdr);
 		fprintf(stderr, "lookup failed!\n");
@@ -734,9 +742,7 @@ query_list(char *str)
 
 	xdr_destroy(&outxdr);
 
-#ifdef _IPC_TYPED_
 	datalen *= BYTES_PER_XDR_UNIT;
-#endif
 
 	xdrmem_create(&inxdr, listbuf, datalen, XDR_DECODE);
 
@@ -762,39 +768,24 @@ query_list(char *str)
 int
 main(int argc, char *argv[])
 {
-	int i, pid, qp, fp;
-	BOOL restarting;
-	BOOL customName;
+	int i, pid, qp, fp, status;
+	BOOL customName, initialStartup;
 	struct rlimit rlim;
-	sys_task_port_type old_lu;
-#ifdef _SHADOW_
-	sys_port_type old_port_privileged;
-	sys_port_type old_port_unprivileged;
-#else
 	sys_port_type old_port;
-#endif
 
 	objc_setMultithreaded(YES);
 	
 	pid = -1;
-	restarting = NO;
 	portName = DefaultName;
 	debugMode = NO;
 	customName = NO;
+	initialStartup = NO;
 
 	server_port = SYS_PORT_NULL;
-#ifdef _SHADOW_
-	server_port_unprivileged = SYS_PORT_NULL;
-	old_port_unprivileged = SYS_PORT_NULL;
-	server_port_privileged = SYS_PORT_NULL;
-	old_port_privileged = SYS_PORT_NULL;
-	shadow_passwords = YES;
-#else
 	old_port = SYS_PORT_NULL;
-#endif
 
-	/* Clean up and re-initialize state on SIGHUP */
-	signal(SIGHUP, handleSIGHUP);
+	signal(SIGHUP, SIG_IGN);
+	signal(SIGTERM, goodbye);
 
 	qp = -1;
 	fp = -1;
@@ -810,10 +801,12 @@ main(int argc, char *argv[])
 			if (qp < 0)
 			{
 				fprintf(stderr, "usage: lookupd -q category [-a key [val ...]] ...\n");
-				exit(1);
+				exit(0);
 			}
 		}
 
+		else if (streq(argv[i], "-startup")) initialStartup = YES;
+
 		else if (streq(argv[i], "-flushcache")) qp = i - 1;
 
 		else if (streq(argv[i], "-statistics")) qp = i - 1;
@@ -831,59 +824,41 @@ main(int argc, char *argv[])
 			debugMode = YES;
 			customName = YES;
 			if (((argc - i) - 1) < 1) 
-			{
+			{				
 				fprintf(stderr,"usage: lookupd -D name\n");
-				exit(1);
+				exit(0);
 			}
 			portName = argv[++i];
 			if (streq(portName, "-")) portName = NULL;
 		}
 
-		else if (streq(argv[i], "-r"))
+		else if (streq(argv[i], "-l"))
 		{
-			if (((argc - i) - 1) < 2) 
+			if (((argc - i) - 1) < 1) 
 			{
-#ifdef _SHADOW_
-				fprintf(stderr,"usage: lookupd -r unprivport privport pid\n");
-#else
-				fprintf(stderr,"usage: lookupd -r port pid\n");
-#endif
-				exit(1);
+				fprintf(stderr,"usage: lookupd -l max_syslog_priority\n");
+				exit(0);
 			}
-
-			restarting = YES;
-
-#ifdef _SHADOW_
-			old_port_unprivileged = (sys_port_type)atoi(argv[++i]);
-			old_port_privileged = (sys_port_type)atoi(argv[++i]);
-#else
-			old_port = (sys_port_type)atoi(argv[++i]);
-#endif
-			pid = atoi(argv[++i]);
+			max_priority = atoi(argv[++i]);
 		}
 
-#ifdef _SHADOW_
-		else if (streq(argv[i], "-u")) shadow_passwords = NO;
-#endif
-
 		else if (streq(argv[i], "-c"))
 		{
 			if (((argc - i) - 1) < 1) 
 			{
 				fprintf(stderr,"usage: lookupd -c source [[domain] path]\n");
-				exit(1);
+				exit(0);
 			}
 
 			i++;
 			if (streq(argv[i], "default")) configSource = configSourceDefault;
-			else if (streq(argv[i], "configd")) configSource = configSourceConfigd;
 			else if (streq(argv[i], "netinfo"))
 			{
 				configSource = configSourceNetInfo;
 				if (((argc - i) - 2) < 1)
 				{
 					fprintf(stderr,"usage: lookupd -c netinfo domain path\n");
-					exit(1);
+					exit(0);
 				}
 
 				configDomain = argv[++i];
@@ -895,7 +870,7 @@ main(int argc, char *argv[])
 				if (((argc - i) - 1) < 1)
 				{
 					fprintf(stderr,"usage: lookupd -c file path\n");
-					exit(1);
+					exit(0);
 				}
 
 				configPath = argv[++i];
@@ -904,17 +879,16 @@ main(int argc, char *argv[])
 			{
 				fprintf(stderr, "Unknown config source.  Must be one of:\n");
 				fprintf(stderr, "    default\n");
-				fprintf(stderr, "    configd\n");
 				fprintf(stderr, "    netinfo \n");
 				fprintf(stderr, "    file\n");
-				exit(1);
+				exit(0);
 			}
 		}
 
 		else if ((qp < 0) && (fp < 0))
 		{
 			fprintf(stderr, "Unknown option: %s\n", argv[i]);
-			exit(1);
+			exit(0);
 		}
 	}
 
@@ -924,13 +898,13 @@ main(int argc, char *argv[])
 		if (i == 0)
 		{
 			fprintf(stderr, "usage: lookupd -q category [-a key [val ...]] ...\n");
-			exit(1);
+			exit(0);
 		}
 
 		if (portName == NULL)
 		{
 			fprintf(stderr, "Can't query without a port\n");
-			exit(1);
+			exit(0);
 		}
 
 		if (i == 1)
@@ -949,54 +923,57 @@ main(int argc, char *argv[])
 		if (i != 3)
 		{
 			fprintf(stderr, "usage: lookupd -f category key val\n");
-			exit(1);
+			exit(0);
 		}
 
 		if (portName == NULL)
 		{
 			fprintf(stderr, "Can't find without a port\n");
-			exit(1);
+			exit(0);
 		}
 
 		query_find(argv[fp + 1], argv[fp + 2], argv[fp + 3]);
 		exit(0);
 	}
-
-	if (restarting && debugMode)
+	
+	if (debugMode)
 	{
-		fprintf(stderr, "Can't restart in debug mode\n");
-		exit(1);
-	}
-
-	if ((!restarting) && (lookupd_port(portName) != SYS_PORT_NULL))
-	{
-		if (debugMode)
+		if (customName)
 		{
-			if (customName)
+			if (getuid() != 0)
 			{
-				fprintf(stderr, "lookupd -D %s is already running!\n",
-					portName);
+				fprintf(stderr,"\nWarning: lookupd -D %s should run as root\n\n", portName);
 			}
-			else
+
+			status = sys_server_status(portName);
+			if (status == SERVER_STATUS_ACTIVE)
 			{
-				fprintf(stderr, "lookupd -d is already running!\n");
+				fprintf(stderr, "lookupd -D %s is already running!\n", portName);
+				exit(0);
 			}
 		}
-		else
+	}
+	else
+	{
+		status = sys_server_status(portName);
+		if (status == SERVER_STATUS_ACTIVE)
 		{
 			fprintf(stderr, "lookupd is already running!\n");
 			system_log(LOG_ERR, "lookupd is already running!\n");
+			exit(0);
 		}
-		exit(1);
+		else if (status == SERVER_STATUS_INACTIVE) initialStartup = YES;
 	}
 
 	if (debugMode)
 	{
+		statistics_enabled = YES;
+
 		lookupd_startup();
 		if (controller == nil)
 		{
 			fprintf(stderr, "controller didn't init!\n");
-			exit(1);
+			exit(0);
 		}
 
 		printf("lookupd version %s (%s)\n", _PROJECT_VERSION_, _PROJECT_BUILD_INFO_);
@@ -1007,71 +984,40 @@ main(int argc, char *argv[])
 		lookupd_shutdown(0);
 	}
 
-	if (restarting)
-	{
-		if (sys_task_for_pid(sys_task_self(), pid, &old_lu) != KERN_SUCCESS)
-		{
-			system_log(LOG_EMERG, "Can't get port for PID %d", pid);
-			exit(1);
-		}
-#ifdef _SHADOW_
-		if (sys_port_extract_receive_right(old_lu, old_port_unprivileged, &server_port_unprivileged)
-			!= KERN_SUCCESS || 
-		    sys_port_extract_receive_right(old_lu, old_port_privileged, &server_port_privileged)
-			!= KERN_SUCCESS || 
-		    port_set_allocate(task_self(), &server_port)
-			!= KERN_SUCCESS || 
-		    port_set_add(task_self(), server_port, server_port_unprivileged)
-			!= KERN_SUCCESS || 
-		    port_set_add(task_self(), server_port, server_port_privileged)
-			!= KERN_SUCCESS)
-#else
-		if (sys_port_extract_receive_right(old_lu, old_port, &server_port)
-			!= KERN_SUCCESS)
-#endif
-		{
-			system_log(LOG_EMERG, "Can't grab port rights");
-			kill(pid, SIGKILL);
-			exit(1);
-		}
-	}
-	else
+	if (initialStartup)
 	{
 		pid = fork();
 		if (pid > 0)
 		{
-			signal(SIGTERM, parentexit);
 			forever sleep(1);
 		}
-
-		detach();
 	}
 
+	detach();
 	closeall();
 
 	if (!debugMode) writepid();
 
-	rlim.rlim_cur = rlim.rlim_max = RLIM_INFINITY;
-	setrlimit(RLIMIT_CORE, &rlim);
-	signal(SIGTERM, goodbye);
+	if (coredump_enabled)
+	{
+		rlim.rlim_cur = rlim.rlim_max = RLIM_INFINITY;
+		setrlimit(RLIMIT_CORE, &rlim);
+	}
 
 	lookupd_startup();
 	if (controller == nil)
 	{
 		system_log(LOG_EMERG, "controller didn't init!");
 		kill(getppid(), SIGTERM);
-		exit(1);
+		exit(0);
 	}
 
-	kill(getppid(), SIGTERM);
+	if (initialStartup) kill(getppid(), SIGTERM);
 
 	[controller serverLoop];
 
 	system_log(LOG_DEBUG, "serverLoop ended");
 
-	/* We get here if the sighup handler got hit. */
-	restart();
-
 	lookupd_shutdown(-1);
 	exit(-1);
 }
