@@ -135,7 +135,7 @@ sys_send_message(sys_msg_header_type *h, sys_msg_timeout_type t)
 void
 sys_port_free(sys_port_type p)
 {
-	mach_port_deallocate(sys_task_self(), p);
+	mach_port_destroy(sys_task_self(), p);
 }
 
 kern_return_t
